from pydantic import BaseModel, root_validator
from typing import Optional


class AuthData(BaseModel):
    client_id: str
    client_secret: str
    authorize_url: str
    access_token_url: str
    refresh_token_url: Optional[str] = None
    scopes: list[str]
    redirect_uri: Optional[str] = None
    state: Optional[str] = None
    access_token: Optional[str] = None
    refresh_token: Optional[str] = None

    @root_validator
    def default_to_access_token(cls, values):
        if not values.get("refresh_token_url"):
            values["refresh_token_url"] = values["access_token_url"]
        return values



from authapi.providers.facebook import facebook


class AuthData(facebook.AuthData):
    scopes: list[str] = [
        "ads_management",
        "ads_read",
        "read_insights",
        "business_management",
    ]



from flask import Flask, render_template, request, redirect, url_for
from requests_oauthlib import OAuth2Session
import os
from typing import Protocol
from authapi.data import AuthData
import pathlib

os.environ["OAUTHLIB_RELAX_TOKEN_SCOPE"] = "1"
os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "1"


class SecretProtocol(Protocol):
    def pull(self) -> dict:
        ...

    def push(self, data: dict) -> None:
        ...


class AuthAPI(Flask):
    def __init__(
        self,
        name: str,
        auth_data: AuthData,
        token_secret: SecretProtocol,
    ):

        template_path = str(pathlib.Path(__file__).parent.absolute() / "templates")
        super().__init__(name, template_folder=template_path)
        self.route("/", methods=["GET"])(self.auth_index)
        self.route("/login", methods=["GET"])(self.login)
        self.route("/authorize", methods=["GET"])(self.authorize)
        self.route("/refresh_token", methods=["GET"])(self.refresh_token)
        self.auth_data = auth_data
        self.auth = OAuth2Session(
            client_id=self.auth_data.client_id,
            redirect_uri=self.auth_data.redirect_uri,
            scope=self.auth_data.scopes,
        )
        self.token_secret = token_secret

    def auth_index(self):
        return render_template(
            "index.html",
            name=self.name,
            state="landing",
            next="login",
        )

    def login(self):
        self.auth.redirect_uri = url_for("authorize", _external=True, _scheme="https")
        authorize_url, self.auth.state = self.auth.authorization_url(
            url=self.auth_data.authorize_url,
            access_type="offline",
            prompt="consent",
        )
        return redirect(authorize_url)

    def authorize(self):
        self.auth.token = self.auth.fetch_token(
            token_url=self.auth_data.access_token_url,
            client_secret=self.auth_data.client_secret,
            authorization_response=request.url,
        )
        self.token_secret.push(self.auth.token)

        return render_template(
            "index.html",
            name=self.name,
            state="authorized",
            next="refresh_token",
        )

    def refresh_token(self):
        self.auth.token = self.token_secret.pull()
        self.auth.token = self.auth.refresh_token(
            token_url=self.auth_data.refresh_token_url,
            client_id=self.auth_data.client_id,
            client_secret=self.auth_data.client_secret,
            refresh_token=self.auth.token["refresh_token"],
        )
        self.token_secret.push(self.auth.token)

        return render_template(
            "index.html",
            name=self.name,
            state="refreshed",
            next="refresh_token",
        )

    def get_token(self):
        self.auth.token = self.token_secret.pull()
        return self.auth.token



from authapi import data


class AuthData(data.AuthData):
    authorize_url: str = "https://www.facebook.com/dialog/oauth"
    access_token_url: str = "https://graph.facebook.com/oauth/access_token"
    scopes: list[str] = [
        "email",
        "public_profile",
    ]



from authapi.providers.facebook import facebook


class AuthData(facebook.AuthData):
    scopes: list[str] = [
        "ads_management",
        "instagram_basic",
        "instagram_manage_insights",
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['authapi',
 'authapi.providers',
 'authapi.providers.facebook',
 'authapi.providers.google']

package_data = \
{'': ['*'], 'authapi': ['templates/*']}

install_requires = \
['Flask>=2.1.2,<3.0.0',
 'cffi>=1.15.0,<2.0.0',
 'pyOpenSSL>=22.0.0,<23.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'requests-oauthlib>=1.3.1,<2.0.0']

setup_kwargs = {
    'name': 'authapi',
    'version': '0.1.4',
    'description': '',
    'long_description': None,
    'author': 'Manuel Castillo',
    'author_email': 'manucalop@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from authapi import data


class AuthData(data.AuthData):
    authorize_url: str = "https://accounts.google.com/o/oauth2/v2/auth"
    access_token_url: str = "https://www.googleapis.com/oauth2/v4/token"
    scopes: list[str] = [
        "openid",
        "https://www.googleapis.com/auth/userinfo.email",
        "https://www.googleapis.com/auth/userinfo.profile",
    ]
